                                                                     
                                                                     
                                                                     
                                             
/**********************************************************
ECE 4760 SP 2011 DJTouch CameraAngle.c
***********************************************************
Tiffany Low twl46
Kritarth Jain kj83

- Memory management using bit-level of char variables 
- Serial port communication using full, hex and reduced printing functions
- Camera image capture program, based off the state machine implemented by Scanalicious 
(http://courses.cit.cornell.edu/ee476/FinalProjects/s2009/dat38/Website/index.html)
- Dynamic image thresholding based max/min intensity of image
- Rotation calculation based on intensity of blobs and a simple thresholding value
- Communication via SPI to LED driver

**********************************************************/

//**********************************************************
//Defines for different run configurations

//define if DEBUG
//#define DEBUG_MODE 1
//define if attempting full data
#define FULLDATA 1
#define PRINTROT 1
//#define HEXPRINT 1
#define REDUCEDPRINT 1
//define if using I2C
#define I2C_COMM	1
//define if using SPI
#define SPI_COMM	1
//define if using UART
#define UART_COMM	1
//define if single capture mode
//#define SINGLE_CAP  1

//**********************************************************
//Hardware Description

// Camera Y at Port A
// Camera datapins
// Port C.0 SCL
// Port C.1 SDA                      
// Port C.2 HREF
// Port C.3 VSYN
// Port D.2 PCLK      
// The usual ports for SPI communication, B4, B5, B6, B7

//**********************************************************
//Includes and Defines
       
#include <avr/io.h>
#include <stdlib.h>
#include <util/delay.h>
#include <avr/interrupt.h>
#include <string.h>
#include <inttypes.h>
#include <stdio.h>

#ifdef I2C_COMM
//I2C communication
#include "i2cmaster.h"
#endif

#ifdef UART_COMM
//UART communication
#include "uart.h"
// UART file descriptor
// putchar and getchar are in uart.c
FILE uart_str = FDEV_SETUP_STREAM(uart_putchar, uart_getchar, _FDEV_SETUP_RW);
#endif


//SPI communication
#define DDR_SPI DDRB
#define DD_MOSI PINB5 
#define DD_MISO PINB6
#define DD_SCK PINB7
#define DD_SS PINB4

//Rotational communication
//#include "matrix.h"

//Camera defines
//image width and height
//QCIF 176x144 format, shrunk window size to 120x120
#define MAXLINES 30
#define MAXPIXELS 30
#define MAXFRAME 15
#define CAMADDR 0xc0
#define SAMPLE 60
#define dataSize (MAXPIXELS/SAMPLE)

#define HREF PORTC & (1<<PINC2)  //PINC.2
#define VSYN PORTC & (1<<PINC3)  //PINC.3
#define PCLK PORTD & (1<<PIND2)  //PIND.2

#define FCPU 16000000UL

char cameraOn;

volatile unsigned char pixelNum;
volatile unsigned char lineNum;
volatile unsigned char bytePos;
volatile unsigned char byteTemp;
volatile unsigned char arrayPos;

//Camera Thresholding
unsigned char THRESHOLD = 180;
unsigned char MINTHRES = 250;
unsigned char MAXTHRES = 0;

#define HARDTHRESLMT 20
#define HARDMINTHRES 100

//State Machine Defines
//Camera State Machine
volatile char syncState;
#define VSYNC     1
#define HREF_RISE    2
#define HREF_FALL   3
//Main State Machine
volatile char commandFlag;
#define HDPROCESSING 1
#define HUPROCESSING 2
#define IDLE         0
#define BUSY		 3

//Processing Defines
/*
Note: ANGLEs start from 0 to 24. ANGLE at 0th bit is the center control bit
*/
char ANGLE[4];
#define NUMANGLE 4
#define aTR 0
#define aTL 1
#define aBL 2
#define aBR 3
#define ANGLETHRESHOLD 1
char TR[MAXFRAME][2], TL[MAXFRAME][2], BR[MAXFRAME][2], BL[MAXFRAME][2];

/*
char at pos 0
(0 1 2 3 4 5 6 7) (0 1 2 3 4 5 6 7)
*/
#define readbit(x,y) ((x << y) & 0x80) >> 7
#define writebit(x,y,b) (b) ? (x | (0x80 >> y)) : (x & ~(0x80 >> y))
#define whichchar(x) (x >> 3) //use this function to tell which char to access in a row
#define whichbit(x) (x & 0x07) //use this function to tell which bit to access in a row
#define whichMATRIX(imr,imc) (imr < 15) ? ((imc < 15) ? aTL : aTR) : ((imc < 15) ? aBL : aBR)

//**********************************************************
// Function Definitions
void turnOffCamera();
void LED_turnon(char a, char b);
void sendData(char line);
void sendNL();
void flicker();
void initialize(void); //all the usual mcu stuff
void datwrite(char MATRIX, char ROW, char COL, char DATA);
char datread(char MATRIX, char ROW, char COL);
void computeRotation(void);
void printrline(char one, char two, char three, char four);
void printangle(char one, char two, char three, char four);
void led_control(void);
char angread(char num);

//**********************************************************
//ISR defines for camera state

//ISR for VSYNC, HREF signals
ISR(PCINT2_vect)
{     

	switch (syncState)
	{
	case VSYNC:
		commandFlag = IDLE;
		syncState = HREF_RISE;

		#ifdef SPI_COMM
			led_control();
		#endif
		//reset line number
		lineNum = 0;
		//turn on HREF ISR
		PCMSK2 = (1<<PCINT18);
		break;
		
	case HREF_RISE:
		//start of ISR-intensive data collection
		syncState = HREF_FALL;
		commandFlag = BUSY;
		//PORTD = 0x00;
		//turn on pixel clock pos edge
		pixelNum = 0;
		bytePos = 0;
		arrayPos = 0;
		EIMSK |= (1<<INT0);
		break;

	case HREF_FALL:    
		//increment line number
		lineNum++;
		//turn off pixel counter            
		EIMSK = 0x00;

		if(lineNum >= MAXLINES)
		{
			syncState = VSYNC;
			//turn on VREF
			PCMSK2 = (1<<PCINT19);
			commandFlag = IDLE; 

			PORTD = (1 << PIND4);			

			#ifdef UART_COMM
			sendNL();
			#endif


			if (MAXTHRES > MINTHRES)
			{
				char temp = (MAXTHRES - MINTHRES);
				if (temp > HARDTHRESLMT)
				{
					int calc = (MAXTHRES + MINTHRES) >> 1;
					//calc = (calc + MAXTHRES) >> 1;
					THRESHOLD = (char) calc;
				}
			}

			//process image
			computeRotation();

			PORTD = 0x00;

		}

		else
		{
			syncState = HREF_RISE;
			commandFlag = HDPROCESSING;
			
			#ifdef UART_COMM
			PORTD = (1 << PIND5);
			#ifdef FULLDATA
			sendData(lineNum);
			#else
			fprintf(stdout, "p");
			#endif
			PORTD = 0x00;
			#endif
		}

		break;
	}
}

///Pxclk (PIND2)
ISR(INT0_vect)
{   

	if (pixelNum < MAXFRAME) {
		if (bytePos == 8)
		{
			//We clear the current byteTemp value to memory
			if (lineNum < MAXFRAME) {
				TL[lineNum][0] = byteTemp;
			}
			else {
				BL[lineNum-15][0] = byteTemp;
			}
			byteTemp = 0x00;
			bytePos = 0;
		}
	}
	if (pixelNum == MAXFRAME) {
		//store data in prev FRAME
		byteTemp = (byteTemp << 1);
		
		if (lineNum < MAXFRAME) {
			TL[lineNum][1] = byteTemp;
		}
		else {
			BL[lineNum-15][1] = byteTemp;
		}
		byteTemp = 0x00;
		bytePos = 0;

	}
	if (pixelNum > MAXFRAME && pixelNum < MAXLINES) {
		if (bytePos == 8)
		{
			//We clear the current byteTemp value to memory
			if (lineNum < MAXFRAME) {
				TR[lineNum][0] = byteTemp;
			}
			else {
				BR[lineNum-15][0] = byteTemp;
			}
			byteTemp = 0x00;
			bytePos = 0;
		}
	}

	if (pixelNum == MAXPIXELS) {
		//end of a line of data
		byteTemp = (byteTemp << 1);
		
		if (lineNum < MAXFRAME) {
			TR[lineNum][1] = byteTemp;
		}
		else {
			BR[lineNum-15][1] = byteTemp;
		}

			byteTemp = 0x00;
			bytePos = 0;

			//Collected enough data on the line, turn off pixel clk
			EIMSK = 0x00;

	}

	//IMAGE Thresholding
	char value = PINA;
	if (value > HARDMINTHRES && value < MINTHRES) 
		MINTHRES = value;
	if (value > MAXTHRES)
		MAXTHRES = value;	

	byteTemp = (byteTemp << 1) | ((value > THRESHOLD) & 0x01);
	
	pixelNum = pixelNum + 1;
	bytePos = bytePos + 1;
}
//**********************************************************
//LED Matrix SPI Communication
void SPI_MasterInit(void)
{
	/* Set MOSI, SS and SCK output, all others input */
	DDR_SPI = (1<<DD_MOSI)|(1<<DD_SCK)|(1<<DD_SS);
	PORTB = PORTB | (1 << DD_SS);
	/* Enable SPI, Master, set clock rate fck/2 = 8MHz*/
	SPCR = (1<<SPE)|(1<<MSTR)|(1<<SPI2X);
}
void SPI_MasterTransmit(char cData)
{
	/* Start transmission */
	SPDR = cData;
	/* Wait for transmission complete */
	while(!(SPSR & (1<<SPIF)));
}

void SPI_init()
{
	//Set Decode Mode
	LED_turnon(0x09, 0x00);
	//Set Intensity
	LED_turnon(0x0A, 0x0F);
	//Set Scan Limit
	LED_turnon(0x0B, 0x07);
	//Set Display Test
	LED_turnon(0x0F, 0x00);
	//Normal operation
	LED_turnon(0x0C, 0x01);
	//Turn off all LEDs
	LED_turnon(0x01, 0x00);
	LED_turnon(0x02, 0x00);
	LED_turnon(0x03, 0x00);
	LED_turnon(0x04, 0x00);
	LED_turnon(0x05, 0x00);
	LED_turnon(0x06, 0x00);
	LED_turnon(0x07, 0x00);
	LED_turnon(0x08, 0x00);
}

void SPI_close()
{
	//Turn off LED display
	LED_turnon(0x0C, 0x00);
}

void LED_turnon(char row, char col)
{
	//Set SS to low
	PORTB = PORTB & ~(1 << DD_SS);
	SPI_MasterTransmit(row);
	SPI_MasterTransmit(col);
	//Set SS to high
	PORTB = PORTB | (1 << DD_SS);
}

//**********************************************************
//Camera I2C Communication
/// I2C read from camera
unsigned char cameraRead(unsigned char regNum)
{
	unsigned char write;
	unsigned char read;
	///Check these if they start to cause trouble
	write = i2c_start(CAMADDR+I2C_WRITE);   
	write = i2c_write(regNum);          
	i2c_stop();                              
	read = i2c_start(CAMADDR+I2C_READ); ;
	read = i2c_readNak();
	i2c_stop();                              
	return read;
}

/// I2C Write to camera
unsigned char cameraWrite(unsigned char regNum, unsigned char data) {
	unsigned char write;
	///Check these if they start to cause trouble
	write = i2c_start(CAMADDR+I2C_WRITE);   
	write = i2c_write(regNum);    
	write = i2c_write(data);          
	i2c_stop();       
	return 0;
}

///its actually already on, we are just deciding to listen to it here
void turnOnCamera()
{

	//PINC3 interrupt set on
	PCICR = (1<<PCIE2);
	PCMSK2 = (1<<PCINT19);//VREF PINC3
	cameraOn = 1;
}

///Ignore the camera's chatter
void turnOffCamera()
{
	///Turn off camera interrupts
	PCICR = 0x00;//&= ~(1<<PCIE2);
	PCMSK2 = 0x00;//&= ~(1<<PCINT19) & ~(1<<PCINT18);

	//turn off pixel counter
	EIMSK &= ~(1<<INT0);
	cameraOn = 0;
}

void initCam(){

	/* TIPS ON GETTING TO WORK 
		- make sure I2C signals are operational
			- must remove programming jumper
			- external pull up added? 4.7k
			- is device powered properly?
	*/

	cameraWrite(0x12, 0x80);  // initiate a soft reset to make sure that all the regsiters are at default values
	_delay_ms(1);
	#ifdef REDUCEDPRINT
	cameraWrite(0x11, 0x06);  // we can speed up pixel clk considerably with reduced print implemented
	#else
	cameraWrite(0x11, 0x13);  
	#endif
	_delay_ms(1);
	cameraWrite(0x12, 0x24);  //COMA: default is x24. YCrCb, disable AWB, 16bit format
	_delay_ms(1);
	cameraWrite(0x14, 0x20);  //COMC: default is x00. turn on QCIF
	_delay_ms(1);
	//Smaller window size: 176 - 120 = 56 or 28 to 148
	//144 - 120 = 24 or 12 to 132
	cameraWrite(0x17,0x56);  
	_delay_ms(1);
	cameraWrite(0x18,0xCE);  
	_delay_ms(1);
	cameraWrite(0x19,0x0E); 
	_delay_ms(1);
	cameraWrite(0x1A,0x86);  
	_delay_ms(1);

	//Exposure
	cameraWrite(0x10, 0x01); //2*HREF*1, shortest exposure time
	_delay_ms(1);
	cameraWrite(0x25, 0xca);
	_delay_ms(1);
	cameraWrite(0x24, 0x01);
	_delay_ms(1);

	///Gain control
	cameraWrite(0x01, 0x00);
	_delay_ms(1);
	cameraWrite(0x02, 0x01);
	_delay_ms(1);

	//Desaturate image
	cameraWrite(0x03, 0x00);
	_delay_ms(1);
	cameraWrite(0x4F, 0x03); //color kill storngest
	_delay_ms(1);
	//Adjust bias to full signal range
	cameraWrite(0x34, 0x00);
	_delay_ms(1);

	//Max contrast, lowest brightness
	cameraWrite(0x05, 0xf0);
	_delay_ms(1);
	cameraWrite(0x06,0xf0);
	_delay_ms(1);


}

//**********************************************************
//LED Control

#define ROW1_REDS 0x92	// 10010010
#define ROW2_REDS 0x49	// 01001001
#define ROW3_REDS 0x24	// 00100100
#define ROW4_REDS 0x92	// 10010010
#define ROW5_REDS 0x49	// 01001001
#define ROW6_REDS 0x24	// 00100100
#define	ROW7_REDS 0x92	// 10010010
#define ROW8_REDS 0x49	// 01001001

#define ROW1_BLUES 0x49	// 01001001
#define ROW2_BLUES 0x24	// 00100100
#define ROW3_BLUES 0x92	// 10010010
#define ROW4_BLUES 0x49	// 01001001
#define ROW5_BLUES 0x24	// 00100100
#define ROW6_BLUES 0x92	// 10010010
#define ROW7_BLUES 0x49	// 01001001
#define ROW8_BLUES 0x24	// 00100100

#define ROW1_GREENS 0x24	// 00100100
#define ROW2_GREENS 0x92	// 10010010
#define ROW3_GREENS 0x49	// 01001001
#define ROW4_GREENS 0x24	// 00100100
#define ROW5_GREENS 0x92	// 10010010
#define ROW6_GREENS 0x49	// 01001001
#define ROW7_GREENS 0x24	// 00100100
#define	ROW8_GREENS 0x92	// 10010010

void allredON(void){
	LED_turnon(0x01,ROW1_REDS);
	LED_turnon(0x02,ROW2_REDS);
	LED_turnon(0x03,ROW3_REDS);
	LED_turnon(0x04,ROW4_REDS);
	LED_turnon(0x05,ROW5_REDS);
	LED_turnon(0x06,ROW6_REDS);
	LED_turnon(0x07,ROW7_REDS);
	LED_turnon(0x08,ROW8_REDS);
}

void allblueON(void){
	LED_turnon(0x01,ROW1_BLUES);
	LED_turnon(0x02,ROW2_BLUES);
	LED_turnon(0x03,ROW3_BLUES);
	LED_turnon(0x04,ROW4_BLUES);
	LED_turnon(0x05,ROW5_BLUES);
	LED_turnon(0x06,ROW6_BLUES);
	LED_turnon(0x07,ROW7_BLUES);
	LED_turnon(0x08,ROW8_BLUES);
}

void allgreenON(void){
	LED_turnon(0x01,ROW1_GREENS);
	LED_turnon(0x02,ROW2_GREENS);
	LED_turnon(0x03,ROW3_GREENS);
	LED_turnon(0x04,ROW4_GREENS);
	LED_turnon(0x05,ROW5_GREENS);
	LED_turnon(0x06,ROW6_GREENS);
	LED_turnon(0x07,ROW7_GREENS);
	LED_turnon(0x08,ROW8_GREENS);
}

unsigned char angle_rows[] = {0x01, 0x01, 0x01,		// 1	2	3
							0x02, 0x02, 0x02, 0x02,	// 4	5	6	7
							0x03, 0x03, 0x03,		// 8	9	10	
							0x04, 0x04, 0x04,		// 11	12	13
							0x05, 0x05, 0x05,		// 14	15	16
							0x06, 0x06, 0x06,		// 17	18	19
							0x07, 0x07, 0x07,		// 20	21	22
							0x08, 0x08};				// 23	24
							
unsigned int angle_cols[] = {0xe0, 0x1c, 0x03,			// 11100000, 00011100, 00000011
							0xc0, 0x30, 0x0c, 0x30,		// 11000000, 00110000, 00001100, 00000011 
							0xc0, 0x38, 0x07,			// 11000000, 00111000, 00000111
							0xe0, 0x1c, 0x03,			// 11100000, 00011100, 00000011
							0xe0, 0x1c, 0x03,			// 11100000, 00011100, 00000011
							0xc0, 0x3c, 0x07,			// 11000000, 00111000, 00000111
							0xe0, 0x1c, 0x03,			// 11100000, 00011100, 00000011
							0xc0, 0x30};					// 11000000, 00110000


int curr_light = 0;

void led_control(void){
	int i;
	unsigned int col_build = 0x00;
	
	if(angread(0) == 1){	// Touching center
		allredON();
	}
	else if(ANGLE[0] == 0 && ANGLE[1] == 0 && ANGLE[2] == 0 && ANGLE[3] == 0){	// None of the angles are being touched
		switch(curr_light){
			case 0:
				allredON();
				curr_light = 1;
				break;
			case 1:
				allblueON();
				curr_light = 2;
				break;
			case 2:
				allgreenON();
				curr_light = 0;
				break;
			default:
				allredON();
				curr_light = 0;
		}
	}

	else{	// Check for touch elsewhere
		for(i=0;i<25;i++){
	  		if(angread(i+1) == 1){
		  		col_build = col_build | angle_cols[i];
			}

	    
	  		if (i == 24 || angle_rows[i] != angle_rows[i+1]){
				LED_turnon(angle_rows[i], col_build);
				col_build = 0x00;
	    	}
		}
	}
}

//**********************************************************
//DATA Processing

/*
0..14 15..29
0+----+----+
.| TL | TR |
14|0.14|0.14|
+----+----+
15| BL | BR |
.|0.14|0.14|
29+----+----+
*/

//mdat is for testing purposes only
void mdatwrite(char IMROW, char IMCOL, char DATA) {
	IMROW = (IMROW < 15) ? IMROW : IMROW - 15;
	IMCOL = (IMCOL < 15) ? IMCOL : IMCOL - 15;
	datwrite(whichMATRIX(IMROW, IMCOL), IMROW, IMCOL, DATA);
}
char mdatread(char IMROW, char IMCOL) {
	IMROW = (IMROW < 15) ? IMROW : IMROW - 15;
	IMCOL = (IMCOL < 15) ? IMCOL : IMCOL - 15;
	return datread(whichMATRIX(IMROW, IMCOL), IMROW, IMCOL);
}

void datwrite(char MATRIX, char ROW, char COL, char DATA) {
	char L = ROW;
	char Wc = whichchar(COL);
	char Wr = whichbit(COL);
	switch(MATRIX)
	{
	case aTL:
		TL[ (int) L][ (int) Wc] = writebit(TL[ (int) L][ (int) Wc], Wr, DATA);
		//fprintf(stdout, "WROTE: %d %02x\n", datread(aTL, ROW, COL), TL[ (int) L][ (int) Wc]);
		break;
	case aTR:
		TR[ (int) L][ (int) Wc] = writebit(TR[ (int) L][ (int) Wc], Wr, DATA);
		break;
	case aBL:
		BL[ (int) L][ (int) Wc] = writebit(BL[ (int) L][ (int) Wc], Wr, DATA);
		break;
	case aBR:
		BR[ (int) L][ (int) Wc] = writebit(BR[ (int) L][ (int) Wc], Wr, DATA);
		break;
	}
}

char datread(char MATRIX, char ROW, char COL) {
	char L = ROW;
	char Wc = whichchar(COL);
	char Wr = whichbit(COL);
	switch(MATRIX)
	{
	case aTL:
		return readbit(TL[ (int) L][ (int) Wc], Wr);
		break;
	case aTR:
		return readbit(TR[ (int) L][ (int) Wc], Wr);
		break;
	case aBL:
		return readbit(BL[ (int) L][ (int) Wc], Wr);
		break;
	case aBR:
		return readbit(BR[ (int) L][ (int) Wc], Wr);
		break;
	}
	return 0; 
}

void angwrite(char num, char DATA) {
	char c = whichchar(num);
	char b = whichbit(num);
	ANGLE[ (int) c] = writebit(ANGLE[ (int) c], b, DATA);
}

char angread(char num) {
	char c = whichchar(num);
	char b = whichbit(num);
	return readbit(ANGLE[ (int) c], b);
}

//**********************************************************
//Rotation Processing Logic Tables

void computeRotation() {
char temp;
temp = datread( aTR, 14, 0)   + datread( aTR, 13, 0)   + datread( aTR, 12, 0)   + datread( aTR, 14, 1)   + datread( aTR, 13, 1)   + datread( aTR, 14, 2)  ; 
temp = temp + datread( aBL, 0, 14)   + datread( aBL, 1, 14)   + datread( aBL, 2, 14)   + datread( aBL, 0, 13)   + datread( aBL, 1, 13)   + datread( aBL, 0, 12)  ; 
temp = temp + datread( aBR, 0, 0)   + datread( aBR, 0, 1)   + datread( aBR, 0, 2)   + datread( aBR, 1, 0)   + datread( aBR, 1, 1)   + datread( aBR, 2, 0)  ; 
temp = temp + datread( aTL, 14, 14)   + datread( aTL, 14, 13)   + datread( aTL, 14, 12)   + datread( aTL, 13, 14)   + datread( aTL, 13, 13)   + datread( aTL, 12, 14)  ; 
angwrite(0,  (temp > ANGLETHRESHOLD) ); 

/*BL Computations*/

temp = datread( aBL, 0, 11)   + datread( aBL, 0, 10)   + datread( aBL, 0, 9)   + datread( aBL, 0, 8)   + datread( aBL, 1, 8)   + datread( aBL, 0, 7)   + datread( aBL, 1, 7)   + datread( aBL, 0, 6)   + datread( aBL, 1, 6)   + datread( aBL, 0, 5)   + datread( aBL, 1, 5)   + datread( aBL, 2, 5)   + datread( aBL, 0, 4)   + datread( aBL, 1, 4)   + datread( aBL, 2, 4)   + datread( aBL, 0, 3)   + datread( aBL, 1, 3)   + datread( aBL, 2, 3)   + datread( aBL, 0, 2)   + datread( aBL, 1, 2)   + datread( aBL, 2, 2)   + datread( aBL, 0, 1)   + datread( aBL, 1, 1)   + datread( aBL, 2, 1)   + datread( aBL, 3, 1)   + datread( aBL, 0, 0)   + datread( aBL, 1, 0)   + datread( aBL, 2, 0)   + datread( aBL, 3, 0)  ; 
angwrite(13,  (temp > ANGLETHRESHOLD) ); 
temp = datread( aBL, 1, 11)   + datread( aBL, 1, 10)   + datread( aBL, 2, 10)   + datread( aBL, 1, 9)   + datread( aBL, 2, 9)   + datread( aBL, 2, 8)   + datread( aBL, 3, 8)   + datread( aBL, 2, 7)   + datread( aBL, 3, 7)   + datread( aBL, 2, 6)   + datread( aBL, 3, 6)   + datread( aBL, 4, 6)   + datread( aBL, 3, 5)   + datread( aBL, 4, 5)   + datread( aBL, 3, 4)   + datread( aBL, 4, 4)   + datread( aBL, 5, 4)   + datread( aBL, 3, 3)   + datread( aBL, 4, 3)   + datread( aBL, 5, 3)   + datread( aBL, 6, 3)   + datread( aBL, 3, 2)   + datread( aBL, 4, 2)   + datread( aBL, 5, 2)   + datread( aBL, 6, 2)   + datread( aBL, 4, 1)   + datread( aBL, 5, 1)   + datread( aBL, 6, 1)   + datread( aBL, 7, 1)   + datread( aBL, 4, 0)   + datread( aBL, 5, 0)   + datread( aBL, 6, 0)   + datread( aBL, 7, 0)  ; 
angwrite(14,  (temp > ANGLETHRESHOLD) ); 
temp = datread( aBL, 1, 12)   + datread( aBL, 2, 11)   + datread( aBL, 3, 10)   + datread( aBL, 3, 9)   + datread( aBL, 4, 9)   + datread( aBL, 4, 8)   + datread( aBL, 5, 8)   + datread( aBL, 4, 7)   + datread( aBL, 5, 7)   + datread( aBL, 6, 7)   + datread( aBL, 5, 6)   + datread( aBL, 6, 6)   + datread( aBL, 7, 6)   + datread( aBL, 5, 5)   + datread( aBL, 6, 5)   + datread( aBL, 7, 5)   + datread( aBL, 8, 5)   + datread( aBL, 6, 4)   + datread( aBL, 7, 4)   + datread( aBL, 8, 4)   + datread( aBL, 9, 4)   + datread( aBL, 7, 3)   + datread( aBL, 8, 3)   + datread( aBL, 9, 3)   + datread( aBL, 10, 3)   + datread( aBL, 7, 2)   + datread( aBL, 8, 2)   + datread( aBL, 9, 2)   + datread( aBL, 10, 2)   + datread( aBL, 11, 2)   + datread( aBL, 8, 1)   + datread( aBL, 9, 1)   + datread( aBL, 10, 1)   + datread( aBL, 11, 1)   + datread( aBL, 12, 1)   + datread( aBL, 8, 0)   + datread( aBL, 9, 0)   + datread( aBL, 10, 0)   + datread( aBL, 11, 0)   + datread( aBL, 12, 0)   + datread( aBL, 13, 0)  ; 
angwrite(15,  (temp > ANGLETHRESHOLD) ); 
temp = datread( aBL, 2, 13)   + datread( aBL, 2, 12)   + datread( aBL, 3, 12)   + datread( aBL, 3, 11)   + datread( aBL, 4, 11)   + datread( aBL, 5, 11)   + datread( aBL, 4, 10)   + datread( aBL, 5, 10)   + datread( aBL, 6, 10)   + datread( aBL, 7, 10)   + datread( aBL, 5, 9)   + datread( aBL, 6, 9)   + datread( aBL, 7, 9)   + datread( aBL, 8, 9)   + datread( aBL, 9, 9)   + datread( aBL, 6, 8)   + datread( aBL, 7, 8)   + datread( aBL, 8, 8)   + datread( aBL, 9, 8)   + datread( aBL, 10, 8)   + datread( aBL, 7, 7)   + datread( aBL, 8, 7)   + datread( aBL, 9, 7)   + datread( aBL, 10, 7)   + datread( aBL, 11, 7)   + datread( aBL, 12, 7)   + datread( aBL, 8, 6)   + datread( aBL, 9, 6)   + datread( aBL, 10, 6)   + datread( aBL, 11, 6)   + datread( aBL, 12, 6)   + datread( aBL, 13, 6)   + datread( aBL, 14, 6)   + datread( aBL, 9, 5)   + datread( aBL, 10, 5)   + datread( aBL, 11, 5)   + datread( aBL, 12, 5)   + datread( aBL, 13, 5)   + datread( aBL, 14, 5)   + datread( aBL, 10, 4)   + datread( aBL, 11, 4)   + datread( aBL, 12, 4)   + datread( aBL, 13, 4)   + datread( aBL, 14, 4)   + datread( aBL, 11, 3)   + datread( aBL, 12, 3)   + datread( aBL, 13, 3)   + datread( aBL, 14, 3)   + datread( aBL, 12, 2)   + datread( aBL, 13, 2)   + datread( aBL, 14, 2)   + datread( aBL, 13, 1)   + datread( aBL, 14, 1)   + datread( aBL, 14, 0)  ; 
angwrite(16,  (temp > ANGLETHRESHOLD) ); 
temp = datread( aBL, 3, 13)   + datread( aBL, 4, 13)   + datread( aBL, 5, 13)   + datread( aBL, 4, 12)   + datread( aBL, 5, 12)   + datread( aBL, 6, 12)   + datread( aBL, 7, 12)   + datread( aBL, 8, 12)   + datread( aBL, 6, 11)   + datread( aBL, 7, 11)   + datread( aBL, 8, 11)   + datread( aBL, 9, 11)   + datread( aBL, 10, 11)   + datread( aBL, 11, 11)   + datread( aBL, 12, 11)   + datread( aBL, 8, 10)   + datread( aBL, 9, 10)   + datread( aBL, 10, 10)   + datread( aBL, 11, 10)   + datread( aBL, 12, 10)   + datread( aBL, 13, 10)   + datread( aBL, 14, 10)   + datread( aBL, 10, 9)   + datread( aBL, 11, 9)   + datread( aBL, 12, 9)   + datread( aBL, 13, 9)   + datread( aBL, 14, 9)   + datread( aBL, 11, 8)   + datread( aBL, 12, 8)   + datread( aBL, 13, 8)   + datread( aBL, 14, 8)   + datread( aBL, 13, 7)   + datread( aBL, 14, 7)  ; 
angwrite(17,  (temp > ANGLETHRESHOLD) ); 
temp = datread( aBL, 3, 14)   + datread( aBL, 4, 14)   + datread( aBL, 5, 14)   + datread( aBL, 6, 14)   + datread( aBL, 7, 14)   + datread( aBL, 8, 14)   + datread( aBL, 9, 14)   + datread( aBL, 10, 14)   + datread( aBL, 11, 14)   + datread( aBL, 12, 14)   + datread( aBL, 13, 14)   + datread( aBL, 14, 14)   + datread( aBL, 6, 13)   + datread( aBL, 7, 13)   + datread( aBL, 8, 13)   + datread( aBL, 9, 13)   + datread( aBL, 10, 13)   + datread( aBL, 11, 13)   + datread( aBL, 12, 13)   + datread( aBL, 13, 13)   + datread( aBL, 14, 13)   + datread( aBL, 9, 12)   + datread( aBL, 10, 12)   + datread( aBL, 11, 12)   + datread( aBL, 12, 12)   + datread( aBL, 13, 12)   + datread( aBL, 14, 12)   + datread( aBL, 13, 11)   + datread( aBL, 14, 11)  ; 
angwrite(18,  (temp > ANGLETHRESHOLD) ); 


/*BR Computations*/

temp = datread( aBR, 3, 0)   + datread( aBR, 4, 0)   + datread( aBR, 5, 0)   + datread( aBR, 6, 0)   + datread( aBR, 6, 1)   + datread( aBR, 7, 0)   + datread( aBR, 7, 1)   + datread( aBR, 8, 0)   + datread( aBR, 8, 1)   + datread( aBR, 9, 0)   + datread( aBR, 9, 1)   + datread( aBR, 9, 2)   + datread( aBR, 10, 0)   + datread( aBR, 10, 1)   + datread( aBR, 10, 2)   + datread( aBR, 11, 0)   + datread( aBR, 11, 1)   + datread( aBR, 11, 2)   + datread( aBR, 12, 0)   + datread( aBR, 12, 1)   + datread( aBR, 12, 2)   + datread( aBR, 13, 0)   + datread( aBR, 13, 1)   + datread( aBR, 13, 2)   + datread( aBR, 13, 3)   + datread( aBR, 14, 0)   + datread( aBR, 14, 1)   + datread( aBR, 14, 2)   + datread( aBR, 14, 3)  ; 
angwrite(19,  (temp > ANGLETHRESHOLD) ); 
temp = datread( aBR, 3, 1)   + datread( aBR, 4, 1)   + datread( aBR, 4, 2)   + datread( aBR, 5, 1)   + datread( aBR, 5, 2)   + datread( aBR, 6, 2)   + datread( aBR, 6, 3)   + datread( aBR, 7, 2)   + datread( aBR, 7, 3)   + datread( aBR, 8, 2)   + datread( aBR, 8, 3)   + datread( aBR, 8, 4)   + datread( aBR, 9, 3)   + datread( aBR, 9, 4)   + datread( aBR, 10, 3)   + datread( aBR, 10, 4)   + datread( aBR, 10, 5)   + datread( aBR, 11, 3)   + datread( aBR, 11, 4)   + datread( aBR, 11, 5)   + datread( aBR, 11, 6)   + datread( aBR, 12, 3)   + datread( aBR, 12, 4)   + datread( aBR, 12, 5)   + datread( aBR, 12, 6)   + datread( aBR, 13, 4)   + datread( aBR, 13, 5)   + datread( aBR, 13, 6)   + datread( aBR, 13, 7)   + datread( aBR, 14, 4)   + datread( aBR, 14, 5)   + datread( aBR, 14, 6)   + datread( aBR, 14, 7)  ; 
angwrite(20,  (temp > ANGLETHRESHOLD) ); 
temp = datread( aBR, 2, 1)   + datread( aBR, 3, 2)   + datread( aBR, 4, 3)   + datread( aBR, 5, 3)   + datread( aBR, 5, 4)   + datread( aBR, 6, 4)   + datread( aBR, 6, 5)   + datread( aBR, 7, 4)   + datread( aBR, 7, 5)   + datread( aBR, 7, 6)   + datread( aBR, 8, 5)   + datread( aBR, 8, 6)   + datread( aBR, 8, 7)   + datread( aBR, 9, 5)   + datread( aBR, 9, 6)   + datread( aBR, 9, 7)   + datread( aBR, 9, 8)   + datread( aBR, 10, 6)   + datread( aBR, 10, 7)   + datread( aBR, 10, 8)   + datread( aBR, 10, 9)   + datread( aBR, 11, 7)   + datread( aBR, 11, 8)   + datread( aBR, 11, 9)   + datread( aBR, 11, 10)   + datread( aBR, 12, 7)   + datread( aBR, 12, 8)   + datread( aBR, 12, 9)   + datread( aBR, 12, 10)   + datread( aBR, 12, 11)   + datread( aBR, 13, 8)   + datread( aBR, 13, 9)   + datread( aBR, 13, 10)   + datread( aBR, 13, 11)   + datread( aBR, 13, 12)   + datread( aBR, 14, 8)   + datread( aBR, 14, 9)   + datread( aBR, 14, 10)   + datread( aBR, 14, 11)   + datread( aBR, 14, 12)   + datread( aBR, 14, 13)  ; 
angwrite(21,  (temp > ANGLETHRESHOLD) ); 
temp = datread( aBR, 1, 2)   + datread( aBR, 2, 2)   + datread( aBR, 2, 3)   + datread( aBR, 3, 3)   + datread( aBR, 3, 4)   + datread( aBR, 3, 5)   + datread( aBR, 4, 4)   + datread( aBR, 4, 5)   + datread( aBR, 4, 6)   + datread( aBR, 4, 7)   + datread( aBR, 5, 5)   + datread( aBR, 5, 6)   + datread( aBR, 5, 7)   + datread( aBR, 5, 8)   + datread( aBR, 5, 9)   + datread( aBR, 6, 6)   + datread( aBR, 6, 7)   + datread( aBR, 6, 8)   + datread( aBR, 6, 9)   + datread( aBR, 6, 10)   + datread( aBR, 7, 7)   + datread( aBR, 7, 8)   + datread( aBR, 7, 9)   + datread( aBR, 7, 10)   + datread( aBR, 7, 11)   + datread( aBR, 7, 12)   + datread( aBR, 8, 8)   + datread( aBR, 8, 9)   + datread( aBR, 8, 10)   + datread( aBR, 8, 11)   + datread( aBR, 8, 12)   + datread( aBR, 8, 13)   + datread( aBR, 8, 14)   + datread( aBR, 9, 9)   + datread( aBR, 9, 10)   + datread( aBR, 9, 11)   + datread( aBR, 9, 12)   + datread( aBR, 9, 13)   + datread( aBR, 9, 14)   + datread( aBR, 10, 10)   + datread( aBR, 10, 11)   + datread( aBR, 10, 12)   + datread( aBR, 10, 13)   + datread( aBR, 10, 14)   + datread( aBR, 11, 11)   + datread( aBR, 11, 12)   + datread( aBR, 11, 13)   + datread( aBR, 11, 14)   + datread( aBR, 12, 12)   + datread( aBR, 12, 13)   + datread( aBR, 12, 14)   + datread( aBR, 13, 13)   + datread( aBR, 13, 14)   + datread( aBR, 14, 14)  ; 
angwrite(22,  (temp > ANGLETHRESHOLD) ); 
temp = datread( aBR, 1, 3)   + datread( aBR, 1, 4)   + datread( aBR, 1, 5)   + datread( aBR, 2, 4)   + datread( aBR, 2, 5)   + datread( aBR, 2, 6)   + datread( aBR, 2, 7)   + datread( aBR, 2, 8)   + datread( aBR, 3, 6)   + datread( aBR, 3, 7)   + datread( aBR, 3, 8)   + datread( aBR, 3, 9)   + datread( aBR, 3, 10)   + datread( aBR, 3, 11)   + datread( aBR, 3, 12)   + datread( aBR, 4, 8)   + datread( aBR, 4, 9)   + datread( aBR, 4, 10)   + datread( aBR, 4, 11)   + datread( aBR, 4, 12)   + datread( aBR, 4, 13)   + datread( aBR, 4, 14)   + datread( aBR, 5, 10)   + datread( aBR, 5, 11)   + datread( aBR, 5, 12)   + datread( aBR, 5, 13)   + datread( aBR, 5, 14)   + datread( aBR, 6, 11)   + datread( aBR, 6, 12)   + datread( aBR, 6, 13)   + datread( aBR, 6, 14)   + datread( aBR, 7, 13)   + datread( aBR, 7, 14)  ; 
angwrite(23,  (temp > ANGLETHRESHOLD) ); 
temp = datread( aBR, 0, 3)   + datread( aBR, 0, 4)   + datread( aBR, 0, 5)   + datread( aBR, 0, 6)   + datread( aBR, 0, 7)   + datread( aBR, 0, 8)   + datread( aBR, 0, 9)   + datread( aBR, 0, 10)   + datread( aBR, 0, 11)   + datread( aBR, 0, 12)   + datread( aBR, 0, 13)   + datread( aBR, 0, 14)   + datread( aBR, 1, 6)   + datread( aBR, 1, 7)   + datread( aBR, 1, 8)   + datread( aBR, 1, 9)   + datread( aBR, 1, 10)   + datread( aBR, 1, 11)   + datread( aBR, 1, 12)   + datread( aBR, 1, 13)   + datread( aBR, 1, 14)   + datread( aBR, 2, 9)   + datread( aBR, 2, 10)   + datread( aBR, 2, 11)   + datread( aBR, 2, 12)   + datread( aBR, 2, 13)   + datread( aBR, 2, 14)   + datread( aBR, 3, 13)   + datread( aBR, 3, 14)  ; 
angwrite(24,  (temp > ANGLETHRESHOLD) ); 



/*TL Computations*/

temp = datread( aTL, 11, 14)   + datread( aTL, 10, 14)   + datread( aTL, 9, 14)   + datread( aTL, 8, 14)   + datread( aTL, 8, 13)   + datread( aTL, 7, 14)   + datread( aTL, 7, 13)   + datread( aTL, 6, 14)   + datread( aTL, 6, 13)   + datread( aTL, 5, 14)   + datread( aTL, 5, 13)   + datread( aTL, 5, 12)   + datread( aTL, 4, 14)   + datread( aTL, 4, 13)   + datread( aTL, 4, 12)   + datread( aTL, 3, 14)   + datread( aTL, 3, 13)   + datread( aTL, 3, 12)   + datread( aTL, 2, 14)   + datread( aTL, 2, 13)   + datread( aTL, 2, 12)   + datread( aTL, 1, 14)   + datread( aTL, 1, 13)   + datread( aTL, 1, 12)   + datread( aTL, 1, 11)   + datread( aTL, 0, 14)   + datread( aTL, 0, 13)   + datread( aTL, 0, 12)   + datread( aTL, 0, 11)  ; 
angwrite(7,  (temp > ANGLETHRESHOLD) ); 
temp = datread( aTL, 11, 13)   + datread( aTL, 10, 13)   + datread( aTL, 10, 12)   + datread( aTL, 9, 13)   + datread( aTL, 9, 12)   + datread( aTL, 8, 12)   + datread( aTL, 8, 11)   + datread( aTL, 7, 12)   + datread( aTL, 7, 11)   + datread( aTL, 6, 12)   + datread( aTL, 6, 11)   + datread( aTL, 6, 10)   + datread( aTL, 5, 11)   + datread( aTL, 5, 10)   + datread( aTL, 4, 11)   + datread( aTL, 4, 10)   + datread( aTL, 4, 9)   + datread( aTL, 3, 11)   + datread( aTL, 3, 10)   + datread( aTL, 3, 9)   + datread( aTL, 3, 8)   + datread( aTL, 2, 11)   + datread( aTL, 2, 10)   + datread( aTL, 2, 9)   + datread( aTL, 2, 8)   + datread( aTL, 1, 10)   + datread( aTL, 1, 9)   + datread( aTL, 1, 8)   + datread( aTL, 1, 7)   + datread( aTL, 0, 10)   + datread( aTL, 0, 9)   + datread( aTL, 0, 8)   + datread( aTL, 0, 7)  ; 
angwrite(8,  (temp > ANGLETHRESHOLD) ); 
temp = datread( aTL, 12, 13)   + datread( aTL, 11, 12)   + datread( aTL, 10, 11)   + datread( aTL, 9, 11)   + datread( aTL, 9, 10)   + datread( aTL, 8, 10)   + datread( aTL, 8, 9)   + datread( aTL, 7, 10)   + datread( aTL, 7, 9)   + datread( aTL, 7, 8)   + datread( aTL, 6, 9)   + datread( aTL, 6, 8)   + datread( aTL, 6, 7)   + datread( aTL, 5, 9)   + datread( aTL, 5, 8)   + datread( aTL, 5, 7)   + datread( aTL, 5, 6)   + datread( aTL, 4, 8)   + datread( aTL, 4, 7)   + datread( aTL, 4, 6)   + datread( aTL, 4, 5)   + datread( aTL, 3, 7)   + datread( aTL, 3, 6)   + datread( aTL, 3, 5)   + datread( aTL, 3, 4)   + datread( aTL, 2, 7)   + datread( aTL, 2, 6)   + datread( aTL, 2, 5)   + datread( aTL, 2, 4)   + datread( aTL, 2, 3)   + datread( aTL, 1, 6)   + datread( aTL, 1, 5)   + datread( aTL, 1, 4)   + datread( aTL, 1, 3)   + datread( aTL, 1, 2)   + datread( aTL, 0, 6)   + datread( aTL, 0, 5)   + datread( aTL, 0, 4)   + datread( aTL, 0, 3)   + datread( aTL, 0, 2)   + datread( aTL, 0, 1)  ; 
angwrite(9,  (temp > ANGLETHRESHOLD) ); 
temp = datread( aTL, 13, 12)   + datread( aTL, 12, 12)   + datread( aTL, 12, 11)   + datread( aTL, 11, 11)   + datread( aTL, 11, 10)   + datread( aTL, 11, 9)   + datread( aTL, 10, 10)   + datread( aTL, 10, 9)   + datread( aTL, 10, 8)   + datread( aTL, 10, 7)   + datread( aTL, 9, 9)   + datread( aTL, 9, 8)   + datread( aTL, 9, 7)   + datread( aTL, 9, 6)   + datread( aTL, 9, 5)   + datread( aTL, 8, 8)   + datread( aTL, 8, 7)   + datread( aTL, 8, 6)   + datread( aTL, 8, 5)   + datread( aTL, 8, 4)   + datread( aTL, 7, 7)   + datread( aTL, 7, 6)   + datread( aTL, 7, 5)   + datread( aTL, 7, 4)   + datread( aTL, 7, 3)   + datread( aTL, 7, 2)   + datread( aTL, 6, 6)   + datread( aTL, 6, 5)   + datread( aTL, 6, 4)   + datread( aTL, 6, 3)   + datread( aTL, 6, 2)   + datread( aTL, 6, 1)   + datread( aTL, 6, 0)   + datread( aTL, 5, 5)   + datread( aTL, 5, 4)   + datread( aTL, 5, 3)   + datread( aTL, 5, 2)   + datread( aTL, 5, 1)   + datread( aTL, 5, 0)   + datread( aTL, 4, 4)   + datread( aTL, 4, 3)   + datread( aTL, 4, 2)   + datread( aTL, 4, 1)   + datread( aTL, 4, 0)   + datread( aTL, 3, 3)   + datread( aTL, 3, 2)   + datread( aTL, 3, 1)   + datread( aTL, 3, 0)   + datread( aTL, 2, 2)   + datread( aTL, 2, 1)   + datread( aTL, 2, 0)   + datread( aTL, 1, 1)   + datread( aTL, 1, 0)   + datread( aTL, 0, 0)  ; 
angwrite(10,  (temp > ANGLETHRESHOLD) ); 
temp = datread( aTL, 13, 11)   + datread( aTL, 13, 10)   + datread( aTL, 13, 9)   + datread( aTL, 12, 10)   + datread( aTL, 12, 9)   + datread( aTL, 12, 8)   + datread( aTL, 12, 7)   + datread( aTL, 12, 6)   + datread( aTL, 11, 8)   + datread( aTL, 11, 7)   + datread( aTL, 11, 6)   + datread( aTL, 11, 5)   + datread( aTL, 11, 4)   + datread( aTL, 11, 3)   + datread( aTL, 11, 2)   + datread( aTL, 10, 6)   + datread( aTL, 10, 5)   + datread( aTL, 10, 4)   + datread( aTL, 10, 3)   + datread( aTL, 10, 2)   + datread( aTL, 10, 1)   + datread( aTL, 10, 0)   + datread( aTL, 9, 4)   + datread( aTL, 9, 3)   + datread( aTL, 9, 2)   + datread( aTL, 9, 1)   + datread( aTL, 9, 0)   + datread( aTL, 8, 3)   + datread( aTL, 8, 2)   + datread( aTL, 8, 1)   + datread( aTL, 8, 0)   + datread( aTL, 7, 1)   + datread( aTL, 7, 0)  ; 
angwrite(11,  (temp > ANGLETHRESHOLD) ); 
temp = datread( aTL, 14, 11)   + datread( aTL, 14, 10)   + datread( aTL, 14, 9)   + datread( aTL, 14, 8)   + datread( aTL, 14, 7)   + datread( aTL, 14, 6)   + datread( aTL, 14, 5)   + datread( aTL, 14, 4)   + datread( aTL, 14, 3)   + datread( aTL, 14, 2)   + datread( aTL, 14, 1)   + datread( aTL, 14, 0)   + datread( aTL, 13, 8)   + datread( aTL, 13, 7)   + datread( aTL, 13, 6)   + datread( aTL, 13, 5)   + datread( aTL, 13, 4)   + datread( aTL, 13, 3)   + datread( aTL, 13, 2)   + datread( aTL, 13, 1)   + datread( aTL, 13, 0)   + datread( aTL, 12, 5)   + datread( aTL, 12, 4)   + datread( aTL, 12, 3)   + datread( aTL, 12, 2)   + datread( aTL, 12, 1)   + datread( aTL, 12, 0)   + datread( aTL, 11, 1)   + datread( aTL, 11, 0)  ; 
angwrite(12,  (temp > ANGLETHRESHOLD) ); 



/*TR Computations*/

temp = datread( aTR, 14, 3)   + datread( aTR, 14, 4)   + datread( aTR, 14, 5)   + datread( aTR, 14, 6)   + datread( aTR, 13, 6)   + datread( aTR, 14, 7)   + datread( aTR, 13, 7)   + datread( aTR, 14, 8)   + datread( aTR, 13, 8)   + datread( aTR, 14, 9)   + datread( aTR, 13, 9)   + datread( aTR, 12, 9)   + datread( aTR, 14, 10)   + datread( aTR, 13, 10)   + datread( aTR, 12, 10)   + datread( aTR, 14, 11)   + datread( aTR, 13, 11)   + datread( aTR, 12, 11)   + datread( aTR, 14, 12)   + datread( aTR, 13, 12)   + datread( aTR, 12, 12)   + datread( aTR, 14, 13)   + datread( aTR, 13, 13)   + datread( aTR, 12, 13)   + datread( aTR, 11, 13)   + datread( aTR, 14, 14)   + datread( aTR, 13, 14)   + datread( aTR, 12, 14)   + datread( aTR, 11, 14)  ; 
angwrite(1,  (temp > ANGLETHRESHOLD) ); 
temp = datread( aTR, 13, 3)   + datread( aTR, 13, 4)   + datread( aTR, 12, 4)   + datread( aTR, 13, 5)   + datread( aTR, 12, 5)   + datread( aTR, 12, 6)   + datread( aTR, 11, 6)   + datread( aTR, 12, 7)   + datread( aTR, 11, 7)   + datread( aTR, 12, 8)   + datread( aTR, 11, 8)   + datread( aTR, 10, 8)   + datread( aTR, 11, 9)   + datread( aTR, 10, 9)   + datread( aTR, 11, 10)   + datread( aTR, 10, 10)   + datread( aTR, 9, 10)   + datread( aTR, 11, 11)   + datread( aTR, 10, 11)   + datread( aTR, 9, 11)   + datread( aTR, 8, 11)   + datread( aTR, 11, 12)   + datread( aTR, 10, 12)   + datread( aTR, 9, 12)   + datread( aTR, 8, 12)   + datread( aTR, 10, 13)   + datread( aTR, 9, 13)   + datread( aTR, 8, 13)   + datread( aTR, 7, 13)   + datread( aTR, 10, 14)   + datread( aTR, 9, 14)   + datread( aTR, 8, 14)   + datread( aTR, 7, 14)  ; 
angwrite(2,  (temp > ANGLETHRESHOLD) ); 
temp = datread( aTR, 13, 2)   + datread( aTR, 12, 3)   + datread( aTR, 11, 4)   + datread( aTR, 11, 5)   + datread( aTR, 10, 5)   + datread( aTR, 10, 6)   + datread( aTR, 9, 6)   + datread( aTR, 10, 7)   + datread( aTR, 9, 7)   + datread( aTR, 8, 7)   + datread( aTR, 9, 8)   + datread( aTR, 8, 8)   + datread( aTR, 7, 8)   + datread( aTR, 9, 9)   + datread( aTR, 8, 9)   + datread( aTR, 7, 9)   + datread( aTR, 6, 9)   + datread( aTR, 8, 10)   + datread( aTR, 7, 10)   + datread( aTR, 6, 10)   + datread( aTR, 5, 10)   + datread( aTR, 7, 11)   + datread( aTR, 6, 11)   + datread( aTR, 5, 11)   + datread( aTR, 4, 11)   + datread( aTR, 7, 12)   + datread( aTR, 6, 12)   + datread( aTR, 5, 12)   + datread( aTR, 4, 12)   + datread( aTR, 3, 12)   + datread( aTR, 6, 13)   + datread( aTR, 5, 13)   + datread( aTR, 4, 13)   + datread( aTR, 3, 13)   + datread( aTR, 2, 13)   + datread( aTR, 6, 14)   + datread( aTR, 5, 14)   + datread( aTR, 4, 14)   + datread( aTR, 3, 14)   + datread( aTR, 2, 14)   + datread( aTR, 1, 14)  ; 
angwrite(3,  (temp > ANGLETHRESHOLD) ); 
temp = datread( aTR, 12, 1)   + datread( aTR, 12, 2)   + datread( aTR, 11, 2)   + datread( aTR, 11, 3)   + datread( aTR, 10, 3)   + datread( aTR, 9, 3)   + datread( aTR, 10, 4)   + datread( aTR, 9, 4)   + datread( aTR, 8, 4)   + datread( aTR, 7, 4)   + datread( aTR, 9, 5)   + datread( aTR, 8, 5)   + datread( aTR, 7, 5)   + datread( aTR, 6, 5)   + datread( aTR, 5, 5)   + datread( aTR, 8, 6)   + datread( aTR, 7, 6)   + datread( aTR, 6, 6)   + datread( aTR, 5, 6)   + datread( aTR, 4, 6)   + datread( aTR, 7, 7)   + datread( aTR, 6, 7)   + datread( aTR, 5, 7)   + datread( aTR, 4, 7)   + datread( aTR, 3, 7)   + datread( aTR, 2, 7)   + datread( aTR, 6, 8)   + datread( aTR, 5, 8)   + datread( aTR, 4, 8)   + datread( aTR, 3, 8)   + datread( aTR, 2, 8)   + datread( aTR, 1, 8)   + datread( aTR, 0, 8)   + datread( aTR, 5, 9)   + datread( aTR, 4, 9)   + datread( aTR, 3, 9)   + datread( aTR, 2, 9)   + datread( aTR, 1, 9)   + datread( aTR, 0, 9)   + datread( aTR, 4, 10)   + datread( aTR, 3, 10)   + datread( aTR, 2, 10)   + datread( aTR, 1, 10)   + datread( aTR, 0, 10)   + datread( aTR, 3, 11)   + datread( aTR, 2, 11)   + datread( aTR, 1, 11)   + datread( aTR, 0, 11)   + datread( aTR, 2, 12)   + datread( aTR, 1, 12)   + datread( aTR, 0, 12)   + datread( aTR, 1, 13)   + datread( aTR, 0, 13)   + datread( aTR, 0, 14)  ; 
angwrite(4,  (temp > ANGLETHRESHOLD) ); 
temp = datread( aTR, 11, 1)   + datread( aTR, 10, 1)   + datread( aTR, 9, 1)   + datread( aTR, 10, 2)   + datread( aTR, 9, 2)   + datread( aTR, 8, 2)   + datread( aTR, 7, 2)   + datread( aTR, 6, 2)   + datread( aTR, 8, 3)   + datread( aTR, 7, 3)   + datread( aTR, 6, 3)   + datread( aTR, 5, 3)   + datread( aTR, 4, 3)   + datread( aTR, 3, 3)   + datread( aTR, 2, 3)   + datread( aTR, 6, 4)   + datread( aTR, 5, 4)   + datread( aTR, 4, 4)   + datread( aTR, 3, 4)   + datread( aTR, 2, 4)   + datread( aTR, 1, 4)   + datread( aTR, 0, 4)   + datread( aTR, 4, 5)   + datread( aTR, 3, 5)   + datread( aTR, 2, 5)   + datread( aTR, 1, 5)   + datread( aTR, 0, 5)   + datread( aTR, 3, 6)   + datread( aTR, 2, 6)   + datread( aTR, 1, 6)   + datread( aTR, 0, 6)   + datread( aTR, 1, 7)   + datread( aTR, 0, 7)  ; 
angwrite(5,  (temp > ANGLETHRESHOLD) ); 
temp = datread( aTR, 11, 0)   + datread( aTR, 10, 0)   + datread( aTR, 9, 0)   + datread( aTR, 8, 0)   + datread( aTR, 7, 0)   + datread( aTR, 6, 0)   + datread( aTR, 5, 0)   + datread( aTR, 4, 0)   + datread( aTR, 3, 0)   + datread( aTR, 2, 0)   + datread( aTR, 1, 0)   + datread( aTR, 0, 0)   + datread( aTR, 8, 1)   + datread( aTR, 7, 1)   + datread( aTR, 6, 1)   + datread( aTR, 5, 1)   + datread( aTR, 4, 1)   + datread( aTR, 3, 1)   + datread( aTR, 2, 1)   + datread( aTR, 1, 1)   + datread( aTR, 0, 1)   + datread( aTR, 5, 2)   + datread( aTR, 4, 2)   + datread( aTR, 3, 2)   + datread( aTR, 2, 2)   + datread( aTR, 1, 2)   + datread( aTR, 0, 2)   + datread( aTR, 1, 3)   + datread( aTR, 0, 3)  ; 
angwrite(6,  (temp > ANGLETHRESHOLD) ); 


}

//**********************************************************
///Send data to PC
void printchar(char x) 
{
	for (int i = 0; i < 8; i ++) {
		if ((x & 0x80) >> 7){
		fprintf(stdout, "1");
		}
		else fprintf(stdout, "0");
		x = x << 1;
	}
}

void sendData(char line) 
{

#ifdef HEXPRINT

	//sends a packet of 90 bits, taking 4.68ms (19200) or 1.56ms (57600)
	if (line < MAXFRAME) {
	fprintf(stdout, "%02x%02x%02x%02x\n", TL[ (int) line][0], TL[ (int) line][1], TR[ (int) line][0], TR[ (int) line][1]);
	}
	else {
	line = line - MAXFRAME;
	fprintf(stdout, "%02x%02x%02x%02x\n", BL[ (int) line][0], BL[ (int) line][1], BR[ (int) line][0], BR[ (int) line][1]);
	}
	//fprintf(stdout, "%02x%02x\n", IMDATA[line][0], IMDATA[line][1]);

#else

#ifdef REDUCEDPRINT
	
	if (line < MAXFRAME) {
	printrline( TL[ (int) line][0], TL[ (int) line][1],TR[ (int) line][0], TR[ (int) line][1]);
	}
	else {
	line = line - MAXFRAME;
	printrline( BL[ (int) line][0], BL[ (int) line][1],BR[ (int) line][0], BR[ (int) line][1]);
	}

	fprintf(stdout, "\n");

#else

	if (line < MAXFRAME) {
	printchar( TL[ (int) line][0]);
	printchar( TL[ (int) line][1]);
	printchar( TR[ (int) line][0]);
	printchar( TR[ (int) line][1]);
	}
	else {
	line = line - MAXFRAME;
	printchar( BL[ (int) line][0]);
	printchar( BL[ (int) line][1]);
	printchar( BR[ (int) line][0]);
	printchar( BR[ (int) line][1]);
	}

	fprintf(stdout, "\n");

#endif

#endif
}
void sendNL()
{
	//sends a packet of 20 bits, taking 1.04ms (19200) or 0.3472 (57600)
	fprintf(stdout, "N");
	#ifdef PRINTROT
	#ifdef HEXPRINT
	for (int i = 0; i < NUMANGLE; i ++)
	{
		fprintf(stdout,"%02x", ANGLE[i]);
	}
	#else

	#ifdef REDUCEDPRINT

		printangle(ANGLE[0], ANGLE[1], ANGLE[2], ANGLE[3]);

	#else

	for (int i = 0; i < NUMANGLE; i ++)
	{
		printchar(ANGLE[i]);
	}
	#endif

	#endif
	#endif
	fprintf(stdout, "\n");
}
//Reduced data version
//Takes advantage of the fact that as long as the first bit is 1, it is guaranteed to not be a control ASCII character
//We send 6 bits of data per character
void printrline(char one, char two, char three, char four){
	char temp = 0;
	//CC111111
	temp = (0x40) | ((one & 0xFC) >> 2);
	fprintf(stdout, "%c", temp);
	//CC112222
	temp = (0x40) | ((one & 0x03) << 4) | ((two & 0xF0) >> 4);
	fprintf(stdout, "%c", temp);
	//CC222333 only 7 bits of data here
	//temp = (0x40) | ((two & 0x0F) << 2) | ((two & 0xC0) >> 6);
	temp = (0x40) | ((two & 0x0E) << 2) | ((three & 0xE0) >> 5);
	fprintf(stdout, "%c", temp);
	//CC333334
	//temp = (0x40) | (three & 0x3F);
	temp = (0x40) | ((three & 0x1F) << 1) | ((four & 0x80) >> 7);
	fprintf(stdout, "%c", temp);
	//CC444444 only 7 bits of data here(only 30 pixels in a line)
	//temp = (0x40) | ((four & 0xFC) >> 2);
	temp = (0x40) | ((four & 0x7E) >> 1);
	fprintf(stdout, "%c", temp);
}

void printangle(char one, char two, char three, char four){
	char temp = 0;
	//CC111111
	temp = (0x40) | ((one & 0xFC) >> 2);
	fprintf(stdout, "%c", temp);
	//CC112222
	temp = (0x40) | ((one & 0x03) << 4) | ((two & 0xF0) >> 4);
	fprintf(stdout, "%c", temp);
	//CC222233 only 7 bits of data here
	temp = (0x40) | ((two & 0x0F) << 2) | ((three & 0xC0) >> 6);
	fprintf(stdout, "%c", temp);
	//CC333333
	temp = (0x40) | (three & 0x3F);
	fprintf(stdout, "%c", temp);
	//CC444444 only 7 bits of data here(only 30 pixels in a line)
	temp = (0x40) | ((four & 0xFC) >> 2);
	fprintf(stdout, "%c", temp);
}


//**********************************************************
//Set it all up
void initialize(void)
{
	#ifdef UART_COMM
	//init the UART -- uart_init() is in uart.c
	uart_init();
	stdout = stdin = stderr = &uart_str;
	#endif	

	///Set PORTA as an input
	DDRA = 0x00;
	///Set External interrupt pins as inputs
	//CHANGED -- if input low not high
	DDRD = DDRD & ~(1 << PIND2);
	DDRC = DDRC & ~(1 << PINC2) & ~(1 << PINC3);

	#ifdef I2C_COMM
	///Set up i2c communcation
	i2c_init();
	#endif

	cameraOn = 0;

	//pixel clock triggers on positive edge
	EICRA |= (1<<ISC01) | (1<<ISC00);

}  

//**********************************************************
//Run the code
int main()
{

	//****************
	//INITIALIZE BLOCK

	initialize();

	#ifdef DEBUG_MODE
	fprintf(stdout, "STARTING\n");
	#endif

	#ifdef I2C_COMM
	initCam();
	_delay_ms(1);

	#ifdef DEBUG_MODE
	fprintf(stdout, "I2C COMPLETE\n");
	#endif

	#endif

	#ifdef SPI_COMM
	SPI_MasterInit();
	SPI_init();

	#ifdef DEBUG_MODE
	fprintf(stdout, "SPI COMPLETE\n");
	#endif

	#endif

	#ifdef DEBUG_MODE
	fprintf(stdout, "INITIALIZE\n");
	#endif

	sei ();

	commandFlag = IDLE;
	syncState = VSYNC;
	turnOnCamera();

	//***************
	//MAIN BLOCK LOOP
	while(1)
	{
		switch (commandFlag)
		{
		case BUSY:
			//DO NOT ADD ANYTHING HERE
			break;
		case IDLE:
			//Completion of one frame, waiting for first HREF signal
			//sendNL();
			#ifdef SPI_COMM 
			//flicker();

			#endif
			break;
		case HUPROCESSING:
			//Completion of one line of data collection
			//flicker();
			break;
		case HDPROCESSING:
			//Long period of idle time
			break;
		}
	}

}
//==================================================
